<?php
/******************************
 * Lake Sumter Community College
 * Copyright 2006
 * Licensed under the Creative Commons
 * "Attribution-NonCommercial-ShareAlike 2.5" license
 *
 * Short summary
 *  http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * Full license information
 *  http://creativecommons.org/licenses/by-nc-sa/2.5/legalcode
 * -----------------------------
 *
 * $Id: $
 * version: 0.3.0 12/13/06
 ******************************/
 
 
class pdodb
{
	private $db;
	private $sqlstrings;		// Array of SQL strings executed or queried
	private $open_time;			// Holder for when the db was connected
	private $close_time;		// Holder for when the db was disconnected
	private $work_time;			// Total CPU Time taken ( minus few flops for tracking )
	private $work_start_time;	// Buffer to keep track of when work starts


	// Constructor
	public function pdodb( $dsn = false )
	{
		if ($dsn)
			return $this->connect($dsn);
		else
			return;
	}
	
	private start_work()
	{
		if ($this->work_start_time == 0.0)
		$this->work_start_time = microtime(true);
	}
	
	private stop_work()
	{
		if ($this->work_start_time != 0.0)
		{
			$this->work_time += microtime(true) - $this->work_start_time;
			$this->work_start_time = 0.0;
		}
	}
	
	private except($message){
		stop_work();
		throw new Exception($message);
	}

	/**
	 * Connect to the database, and select it if $name is provided
	 *
	 * @param string $dsn database connection string
	 * @param string $db_prefix database prefix
	 * @return bool
	 */
	public function connect( $dsn )
	{
		start_work();
		$sqlstrings = null;
		$this->closeDb();
		$this->db = new PDO($dsn);

		if( $this->db )
		{
			$this->open_time = microtime(true);
			stop_work();
			return true;
		}
		else
		{
			stop_work();
			return false;
		}
		
	}
	
	/**
	 * Get all queries/statements executed
	 *
	 * @return string[]
	 */
	public function getSQLStrings()
	{
		return $this->sqlstrings;
	}

	/**
	 * Check last operation on the database handle or stmt for any errors
	 *
	 * @param PDOStatement $stmt
	 * @return void
	 */
	public function checkError( $stmt = null )
	{
		start_work();
		if( !$this->db )except('Database not connected.');
		if (!$stmt)
		{
			if ($this->db->errorCode() != '00000')
			{
				$error = "<pre>".var_export ( $this->db->errorInfo(), true )."</pre>";
				except($error);
			}
		}
		else
		{
			if ($stmt->errorCode() != '00000')
			{
				$error = "<pre>".var_export ( $stmt->errorInfo(), true )."</pre>";
				except($error);
			}
		}
		stop_work();
	}

	/**
	 * Frees up the connection to the server so that other SQL stmts may be issued, 
	 * but leaves the stmt in a state that enables it to be executed again.
	 *
	 * @param string $sql
	 * @return bool
	 */
	public function free( $stmt )
	{
		start_work();
		if (!$stmt)
		{
			except('Invalid statement passed');
		}
		$bool = $stmt->closeCursor();
		stop_work();
		return $bool;
	}

	/**
	 * Prepares a stmt for execution and returns a stmt object
	 *
	 * @param string $sql
	 * @return PDOStatement
	 */
	public function prepare( $sql )
	{
		start_work();
		if( !$this->db )except('Database not connected.');
		$this->sqlstrings[] = $sql;
		$stmt = $this->db->prepare($sql);
		stop_work();
		return $stmt;
	}
	
	/**
	 * Binds a PHP variable to a corresponding named or question mark placeholder in the SQL stmt that 
	 * was use to prepare the stmt. Unlike PDOstmt::bindValue(), the variable is bound as a 
	 * reference and will only be evaluated at the time that PDOstmt::execute() is called.
	 *
	 * @param PDOstmt $stmt
	 * @param mixed $parameter 	( Parameter identifier. For a prepared stmt using named placeholders, this will be a parameter name of the form :name. For a prepared stmt using question mark placeholders, this will be the 1-indexed position of the parameter. )
	 * @param mixed $var 		( Name of the PHP variable to bind to the SQL stmt parameter. )
	 * @param int $type 		( Explicit data type for the parameter using the PDO::PARAM_* constants. To return an INOUT parameter from a stored procedure, use the bitwise OR operator to set the PDO::PARAM_INPUT_OUTPUT bits for the data_type parameter. )
	 * @param int $length 		( Length of the data type. To indicate that a parameter is an OUT parameter from a stored procedure, you must explicitly set the length. )
	 * @param int $options 		( driver_options )
	 * @return void
	 */
	public function bindParam( $stmt, $parameter, &$var, $type = false, $length = false, $options = false)
	{
		start_work();
		if( !$this->db )except('Database not connected.');
		if ($options && $length && $type)
		{
			$stmt->bindParam($parameter, $var, $type, $length, $options);
		}
		elseif(!$options && $length && $type)
		{
			$stmt->bindParam($parameter, $var, $type, $length);
		}
		elseif(!$options && !$length && $type)
		{
			$stmt->bindParam($parameter, $var, $type);		
		}
		elseif(!$options && !$length && !$type)
		{
			$stmt->bindParam($parameter, $var);
		}
		else
		{
			$stmt->bindParam($parameter, $var);
		}
		stop_work();
	}
	
	/**
	 * arranges to have a particular variable bound to a given column in the stmt-set from a query. 
	 * Each call to fetch_assoc, fetch_both, or fetch_obj will update all the variables that are bound to columns.
	 *
	 * @param PDOstmt $stmt
	 * @param mixed $column ( Number of the column (1-indexed) or name of the column in the stmt set. If using the column name, be aware that the name should match the case of the column, as returned by the driver. )
	 * @param mixed $var ( Name of the PHP variable to which the column will be bound. )
	 * @param int $type ( Data type of the parameter, specified by the PDO::PARAM_* constants. )
	 * @return void
	 */
	public function bindColumn( $stmt, $col, &$var, $type = false )
	{
		start_work();
		if( !$this->db )except('Database not connected.');
		if (!$type)
		{
			$stmt->bindColumn($col, $var);
		}
		else
		{
			$stmt->bindColumn($col, $var, $type);
		}
		stop_work();
	}
	
	/**
	 * Binds a value to a corresponding named or question mark placeholder in the SQL stmt that was use to prepare the stmt.
	 *
	 * @param PDOstmt $stmt
	 * @param mixed $parameter ( Parameter identifier. For a prepared stmt using named placeholders, this will be a parameter name of the form :name. For a prepared stmt using question mark placeholders, this will be the 1-indexed position of the parameter. )
	 * @param mixed $val ( The value to bind to the parameter. )
	 * @param int $type ( Data type of the parameter, specified by the PDO::PARAM_* constants. )
	 * @return void
	 */
	public function bindValue( $stmt, $parameter, $val, $type = false )
	{
		start_work();
		if( !$this->db )except('Database not connected.');
		if (!type)
		{
			$stmt->bindValue($parameter, $var);
		}
		else
		{
			$stmt->bindValue($parameter, $var, $type);
		}
		stop_work();
	}
	
	
	
	/**
	 * Execute the prepared stmt.
	 *
	 * @param PDOstmt $stmt
	 * @param array $input ( An array of values with as many elements as there are bound parameters in the SQL stmt being executed. )
	 * @return bool
	 */
	public function execute( $stmt, $input = false )
	{
		start_work();
		if( !$this->db )except('Database not connected.');
		if (!$input)
		{
			$success = $stmt->execute();
			$this->sqlstrings[] = "[ execute() ]";
			$this->checkError($stmt);
			stop_work();
			return $success;
		}
		else
		{
			$success = $stmt->execute($input);
			$this->sqlstrings[] = "[ execute( <blockquote>".var_export($input,true)."</blockquote> ) ]";
			$this->checkError($stmt);
			stop_work();
			return $success;
		}
	}
	
	/**
	 * Initiates a transaction
	 *
	 * @return bool
	 */
	public function beginTransaction()
	{
		start_work();
		if( !$this->db )except('Database not connected.');
		$bool = $this->db->beginTransaction();
		stop_work();
		return $bool;
	}
	
	/**
	 * Rolls back a transaction
	 *
	 * @return bool
	 */
	public function rollBack()
	{
		start_work();
		if( !$this->db )except('Database not connected.');
		$bool = $this->db->rollBack();
		stop_work();
		return $bool;
	}
	
	/**
	 * Commits a transaction
	 *
	 * @return bool
	 */
	public function commit()
	{
		start_work();
		if( !$this->db )except('Database not connected.');
		stop_work();
		return $this->db->commit();
	}
	
	/**
	 * Execute an SQL statement and return the number of affected rows
	 *
	 * @param string $sql
	 * @return int
	 */
	public function exec( $sql )
	{
		start_work();
		if( !$this->db )except('Database not connected.');
		$count = $this->db->exec($sql);
		$this->sqlstrings[] = $sql;
		$this->checkError();
		stop_work();
		return $count;
	}
	
	/**
	 * retrieve some database information
	 *
	 * @return string[]
	 */
	public function getAttributes()
	{
		start_work();
		$attributes = array(
		   "AUTOCOMMIT", "ERRMODE", "CASE", "CLIENT_VERSION", "CONNECTION_STATUS",
		   "ORACLE_NULLS", "PERSISTENT", "PREFETCH", "SERVER_INFO", "SERVER_VERSION",
		   "TIMEOUT"
		);
		$attrArr = array();
		foreach ($attributes as $val) {
		    @$attrArr[$val] = $this->db->getAttribute(constant("PDO::ATTR_$val"));
		}
		stop_work();
		return $attrArr;
	}
	
	/**
	 * Set an attribute
	 *
	 * @param int $attribute
	 * @param mixed $value
	 * @return bool
	 */
	public function setAttribute( $attribute, $value )
	{
		start_work();
		if( !$this->db )except('Database not connected.');
		$bool = $this->db->setAttribute($sql);
		stop_work();
		return $bool;
	}
	
	/**
	 * retrieve all available database drivers
	 *
	 * @return string[]
	 */
	public function getAvailableDrivers()
	{
		start_work();
		$return = PDO::getAvailableDrivers();
		stop_work();
		return $return;
	}
	

	/**
	 * for running simple queries
	 *
	 * @param string $sql
	 * @return mixed $stmt handle
	 */
	public function query( $sql )
	{
		start_work();
		if( !$this->db )except('Database not connected.');
		$this->sqlstrings[] = $sql;

		if (stripos($sql, 'select') !== false)
		{
			$stmt = $this->db->query($sql);
			$this->checkError($stmt);
			stop_work();
			return $stmt;
		}
		else
		{
			$count = $this->db->exec($sql);
			$this->checkError();
			stop_work();
			return $count;
		}
	}
	

	/**
	 * Returns the ID of the last inserted row or sequence value
	 *
	 * @param string $name
	 * @return string
	 */
	public function lastInsertId( $name = false )
	{
		start_work();
		if( !$this->db )except('Database not connected.');
		if ($name)
		{
			$out = $this->db->lastInsertId($name);
			stop_work();
			return $out;
		}
		else
		{
			$out = $this->db->lastInsertId();
			stop_work();
			return $out;
		}
	}
	
	
	/**
	 * Front-end for SQL num affected rows
	 *
	 * @param PDOStatement $stmt
	 * @return int $num_affected number of rows affected -1 for invalid stmt
	 */
	public function num_affected( $stmt )
	{
		start_work();
		if (!$stmt)
		{
			except('Invalid statement passed');
		}
		$out = $stmt->rowCount();
		stop_work();
		return $out;
	}

	/**
	 * Front-end for SQL_fetch_assoc
	 *
	 * @param PDOStatement $stmt handle
	 * @returns an array indexed by column name as returned in your stmt set
	 */
	public function fetch_assoc( $stmt )
	{
		start_work();
		if (!$stmt)
		{
			except('Invalid statement passed');
		}
		$out = $stmt->fetch(PDO::FETCH_ASSOC);
		stop_work();
		return $out;
	}


	/**
	 * Front-end for SQL_fetch_array
	 *
	 * @param PDOStatement $stmt handle
	 * @returns an array indexed by both column name and 0-indexed column number as returned in your stmt set
	 */
	public function fetch_both( $stmt )
	{
		start_work();
		if (!$stmt)
		{
			except('Invalid statement passed');
		}
		$out = $stmt->fetch(PDO::FETCH_BOTH);
		stop_work();
		return $out;
	}
	
	/**
	 * Front-end for SQL_fetch_object
	 *
	 * @param PDOStatement $stmt handle
	 * @return object current db row
	 */
	public function fetch_obj( $stmt )
	{
		start_work();
		if (!$stmt)
		{
			except('Invalid statement passed');
		}
		$out = $stmt->fetch(PDO::FETCH_OBJ);
		stop_work();
		return $out;
	}
	
	/**
	 * assign the values of the columns in the result set to the PHP variables to which they were bound with the PDOStatement::bindParam() or PDOStatement::bindColumn() methods.
	 *
	 * @param PDOStatement $stmt handle
	 * @return object current db row
	 */
	public function fetch_bound( $stmt )
	{
		start_work();
		if (!$stmt)
		{
			except('Invalid statement passed');
		}
		$out = $stmt->fetch(PDO::FETCH_BOUND);
		stop_work();
		return $out;
	}
	
	/**
	 * Returns an array containing all of the result set rows
	 *
	 * @param PDOStatement $stmt handle
	 * @param int $style
	 * @param int $column_index
	 * @return array
	 */
	public function fetchAll( $stmt, $style = PDO::FETCH_BOTH, $column_index = 0 )
	{
		start_work();
		if (!$stmt)
		{
			except('Invalid statement passed');
		}
		$out = $stmt->fetchAll($style, $column_index);
		stop_work();
		return $out;
	}
	
	/**
	 * Returns a single column from the next row of a result set
	 *
	 * @param PDOStatement $stmt handle
	 * @param int $column_index ( 0-indexed number of the column you wish to retrieve from the row. If no value is supplied, PDOStatement::fetchColumn()  fetches the first column. )
	 * @return string
	 */
	public function fetchColumn( $stmt, $column_index = 0 )
	{
		start_work();
		if (!$stmt)
		{
			except('Invalid statement passed');
		}
		$out = $stmt->fetchColumn($column_index);
		stop_work();
		return $out;
	}
	
	/**
	 * Returns metadata for a column in a result set
	 *
	 * @param PDOStatement $stmt handle
	 * @param int $column_index ( The 0-indexed column in the result set. )
	 * @return array
	 */
	public function getColumnMeta( $stmt, $column_index = 0 )
	{
		start_work();
		if (!$stmt)
		{
			except('Invalid statement passed');
		}
		$out = $stmt->getColumnMeta($column_index);
		stop_work();
		return $out;
	}
	
	/**
	 * Advances to the next rowset in a multi-rowset statement handle
	 *
	 * @param PDOStatement $stmt handle
	 * @return bool
	 */
	public function nextRowset( $stmt )
	{
		start_work();
		if (!$stmt)
		{
			except('Invalid statement passed');
		}
		$out = $stmt->nextRowset();
		stop_work();
		return $out;
	}
	

	/**
	 * Returns the number of columns in the stmt set
	 *
	 * @param int $stmt handle
	 * @return int number of columns in stmt
	 */
	public function columnCount( $stmt )
	{
		start_work();
		if (!$stmt)
		{
			except('Invalid statement passed');
		}
		$out = $stmt->columnCount();
		stop_work();
		return $out;
	}
	
	/**
	 * Front-end for SQL_num_rows
	 *
	 * @param int $stmt handle
	 * @return int number of rows in stmt
	 */
	public function num_rows( $stmt )
	{
		start_work();
		$out = count($stmt->fetchAll());
		stop_work();		
		return $out;
	}


	/**
	 * Front-end to escape string for safe inserting
	 *
	 * @param string $string
	 * @return string escaped
	 */
	public function escape( $string )
	{
		return mysql_real_escape_string( $string );
	}
	
	/**
	 * Quotes a string for use in a query.
	 *
	 * @param string $string
	 * @return string quoted
	 */
	public function quote( $string )
	{
		return $this->db->quote( $string );
	}
	

	/**
	 * Closes the DB connection
	 *
	 * @return bool
	 */
	public function closeDb()
	{
		start_work();
		if( $this->db )
		{
			$this->db = null;
			$this->close_time = microtime(true);
			stop_work();
			return true;
		}
		else
		{
			stop_work();
			return false;
		}
	}
	
	/**
	 * Get total time between when the database was opened and when it was closed (or now), or the CPU time used to perform any work.
	 *
	 * @return float
	 */
	public function benchmark($total = false)
	{
		if ($total)
		{
			if( $this->db )
			{
				return microtime(true) - $this->open_time;
			}
			else
			{
				return $this->close_time - $this->open_time;
			}
		}
		else
		{
			return $this->work_time;
		}
	}
} //-END CLASS

?>
